%############################################################################
%#
%#                          function mandhala_2D
%#
%#    calculates ONLY the 2D-GEOMETRY of single mandhala rings. Multiple rings    
%#    are connected by calling procedure, magnetic properties are also
%#    defined somewhere else
%#    
%#   input: 
%#     type = 0  old block geometry
%#          = 1  new polygonal geometry
%#   radius = [inner_radius, center_radius, outer_radius,size of magnet] 
%#            those which are set to 0 are ignored. 
%#        N = number of magnets  
%#        k = Halbach k (+1 = inner Dipole, -2=outer quadrupole)
%#     dist = distance between magnets
%#  N_phase = additional phase in magnet position [in degree]
%#    ring0 = (optional) already existing ring structure, to pipe other
%#             settings
%#    
%#   output:
%#     ring = structure
%#     ring.N = number of magnets in this ring
%#  ring.type = type from input
%#     ring.k = k from input
%#  ring.dist = dist from input
%#  ring.N_phase = N_phase from input
%#     ring.n = number of magnet edges
%#     ring.r = radii of ring, vector = [inner_radius, center_radius, outer_radius, size of magnet] 
%#     ring.c = centers of magnets, vector of length (N,2), 
%#              last contains x,y, coordinates
%#     ring.p = edges of magnets, vector of length (N,n,2), 
%#              last contains x,y, coordinates
%#     ring.m = angle of magnetization vector in each magnet
%#
%#
%#
%#      (c) P. Blmler 3/07
%############################################################################
%----------------------------------------------------------------------------
%  version 1.0 PB 23/3/07    (please change this when code is altered)
%----------------------------------------------------------------------------

function ring=mandhala_2D(type,radius,N,k,dist,N_phase,ring0)
if nargin==6
    ring0=0;
else
    ring=ring0;
end
ring.type=type;
ring.N = N;
ring.k=k;
ring.dist=dist;
ring.N_phase=N_phase;
ring.c=zeros(ring.N,2);

i=0:N-1;
alpha1=2*pi/N;
alpha = i*alpha1+N_phase*pi/180;  %angle of each magnet 
beta =-(1+k)*alpha+pi/2;  %angle of each magnetization
ring.m = beta;
ring.type=type;
if type==1 %new geometry
  ring.n=N/2;
  ring.p=zeros(ring.N,ring.n,2);
 
  if radius(1)~=0 %inner radius given
     if mod(ring.n,2)==0 %n/2 = even
       radius(2)=radius(1)/(1-sin(alpha1)/2/cos(alpha1));
       a=radius(2)*sin(alpha1)^2/cos(alpha1);
     elseif mod(ring.n,2)==1 %n/2 = odd
       radius(2)=radius(1)/(1-2*sin(alpha1/2)/(cos(alpha1)+1));
       a=4*radius(2)*sin(alpha1)/(cos(alpha1)+1)*sin(alpha1/2);
     else
       disp('ERROR: number of magents has to be a multiple of 2');
       return
     end
     ax=a/2/sin(alpha1);
     radius(3)=radius(2)+ax;
     radius(4)=ax-dist/2;
  elseif radius(2)~=0 %center radius given
     if mod(ring.n,2)==0 %n/2 = even
       a=radius(2)*sin(alpha1)^2/cos(alpha1);
     elseif mod(ring.n,2)==1 %n/2 = odd
       a=4*radius(2)*sin(alpha1)/(cos(alpha1)+1)*sin(alpha1/2);
     else
       disp('ERROR: number of magents has to be a multiple of 2');
       return
     end
     ax=a/2/sin(alpha1);
     radius(1)=radius(2)-ax;
     radius(3)=radius(2)+ax;
     radius(4)=ax-dist/2;
  elseif radius(3)~=0  %outer radius given 
     if mod(ring.n,2)==0 %n/2 = even
       radius(2)=radius(3)/(1+sin(alpha1)/2/cos(alpha1));
       a=radius(2)*sin(alpha1)^2/cos(alpha1);
     elseif mod(ring.n,2)==1 %n/2 = odd
       radius(2)=radius(3)/(1+sin(alpha1/2)/(cos(alpha1)+1));
       a=4*radius(2)*sin(alpha1)/(cos(alpha1)+1)*sin(alpha1/2);
     else
       disp('ERROR: number of magents has to be a multiple of 2');
       return
     end
     ax=a/2/sin(alpha1);
     radius(1)=radius(2)-ax;
     radius(4)=ax-dist/2;
  else %magnet size given
      ax=radius(4)/cos(pi/ring.n)+dist/2;
      a=ax*2*sin(alpha1);
      if mod(ring.n,2)==0 %n/2 = even
       radius(2)=a/sin(alpha1)^2*cos(alpha1);
     elseif mod(ring.n,2)==1 %n/2 = odd
       radius(2)=a/4/sin(alpha1)*(cos(alpha1)+1)*sin(alpha1/2);
     else
       disp('ERROR: number of magents has to be a multiple of 2');
       return
     end
     radius(1)=radius(2)-ax;
     radius(3)=radius(2)+ax;
  end
  ring.r=radius;
  ring.p=zeros(ring.N,ring.n,2);
  j=1:ring.n;
  xi=(2*j+1)*alpha1;
  ring.c(:,1)=radius(2)*sin(alpha);
  ring.c(:,2)=radius(2)*cos(alpha);
  
  for i=1:ring.N
      for j=1:ring.n
        ring.p(i,j,1)=ring.c(i,1)+cos(N_phase*pi/90)*ring.r(4)*sin(xi(j))+sin(N_phase*pi/90)*ring.r(4)*cos(xi(j));   %x-coordinate magnet i edge j
        ring.p(i,j,2)=ring.c(i,2)-sin(N_phase*pi/90)*ring.r(4)*sin(xi(j))+cos(N_phase*pi/90)*ring.r(4)*cos(xi(j));   %y-coordinate magnet i edge j
      end
  end
  
  
else
  ring.n=4;
  ring.p=zeros(ring.N,ring.n,2);
  Xi=(cos(2*pi/N)-sin(2*pi/N)-sqrt(2)*sin(pi/4-4*pi/N))/(2*cos(pi/4-4*pi/N)+sqrt(2));
  if radius(1)~=0 %inner radius given
     radius(2)=radius(1)/(1-sqrt(2)*Xi);
     radius(3)=radius(2)*(1+sqrt(2)*Xi);
     radius(4)=radius(2)*2*Xi-dist/2;
  elseif radius(2)~=0 %center radius given
     radius(1)=radius(2)*(1-sqrt(2)*Xi);
     radius(3)=radius(2)*(1+sqrt(2)*Xi);
     radius(4)=radius(2)*2*Xi-dist/2;
  elseif radius(3)~=0  %outer radius given 
     radius(2)=radius(3)/(1+sqrt(2)*Xi);
     radius(1)=radius(2)*(1-sqrt(2)*Xi);
     radius(4)=radius(2)*2*Xi-dist/2;
  else %magnet size given
     radius(2)=radius(4)/2/Xi+dist/2;
     radius(1)=radius(2)*(1-sqrt(2)*Xi);
     radius(3)=radius(2)*(1+sqrt(2)*Xi);
  end
  ring.r=radius;
  ring.p=zeros(ring.N,ring.n,2);
  xi=pi/4-2*alpha;
  ring.c(:,1)=radius(2)*sin(alpha);
  ring.c(:,2)=radius(2)*cos(alpha);
  
  for i=1:ring.N
      ring.p(i,1,1)=ring.c(i,1)+ring.r(4)/sqrt(2)*cos(xi(i));   %x-coordinate magnet 1
      ring.p(i,1,2)=ring.c(i,2)+ring.r(4)/sqrt(2)*sin(xi(i));   %x-coordinate magnet 1
      ring.p(i,2,1)=ring.c(i,1)-ring.r(4)/sqrt(2)*sin(xi(i));   %x-coordinate magnet 2
      ring.p(i,2,2)=ring.c(i,2)+ring.r(4)/sqrt(2)*cos(xi(i));   %x-coordinate magnet 2
      ring.p(i,3,1)=ring.c(i,1)-ring.r(4)/sqrt(2)*cos(xi(i));   %x-coordinate magnet 3
      ring.p(i,3,2)=ring.c(i,2)-ring.r(4)/sqrt(2)*sin(xi(i));   %x-coordinate magnet 3
      ring.p(i,4,1)=ring.c(i,1)+ring.r(4)/sqrt(2)*sin(xi(i));   %x-coordinate magnet 4
      ring.p(i,4,2)=ring.c(i,2)-ring.r(4)/sqrt(2)*cos(xi(i));   %x-coordinate magnet 4
  end

end
    

